@@ -21,8 +21,8 @@
 #include "hash.h"
 
 struct client_eta {
-	struct jobs_eta eta;
 	unsigned int pending;
+	struct jobs_eta eta;
 };
 
 struct fio_client {
@@ -267,7 +267,7 @@ int fio_client_add(const char *hostname, void **cookie)
 static int fio_client_connect_ip(struct fio_client *client)
 {
 	struct sockaddr *addr;
-	fio_socklen_t socklen;
+	socklen_t socklen;
 	int fd, domain;
 
 	if (client->ipv6) {
@@ -304,7 +304,7 @@ static int fio_client_connect_ip(struct fio_client *client)
 static int fio_client_connect_sock(struct fio_client *client)
 {
 	struct sockaddr_un *addr = &client->addr_un;
-	fio_socklen_t len;
+	socklen_t len;
 	int fd;
 
 	memset(addr, 0, sizeof(*addr));
@@ -595,8 +595,9 @@ static void convert_ts(struct thread_stat *dst, struct thread_stat *src)
 	dst->groupid	= le32_to_cpu(src->groupid);
 	dst->pid	= le32_to_cpu(src->pid);
 	dst->members	= le32_to_cpu(src->members);
+	dst->unified_rw_rep	= le32_to_cpu(src->unified_rw_rep);
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		convert_io_stat(&dst->clat_stat[i], &src->clat_stat[i]);
 		convert_io_stat(&dst->slat_stat[i], &src->slat_stat[i]);
 		convert_io_stat(&dst->lat_stat[i], &src->lat_stat[i]);
@@ -628,11 +629,11 @@ static void convert_ts(struct thread_stat *dst, struct thread_stat *src)
 		dst->io_u_lat_m[i]	= le32_to_cpu(src->io_u_lat_m[i]);
 	}
 
-	for (i = 0; i < 2; i++)
+	for (i = 0; i < DDIR_RWDIR_CNT; i++)
 		for (j = 0; j < FIO_IO_U_PLAT_NR; j++)
 			dst->io_u_plat[i][j] = le32_to_cpu(src->io_u_plat[i][j]);
 
-	for (i = 0; i < 3; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		dst->total_io_u[i]	= le64_to_cpu(src->total_io_u[i]);
 		dst->short_io_u[i]	= le64_to_cpu(src->short_io_u[i]);
 	}
@@ -640,7 +641,7 @@ static void convert_ts(struct thread_stat *dst, struct thread_stat *src)
 	dst->total_submit	= le64_to_cpu(src->total_submit);
 	dst->total_complete	= le64_to_cpu(src->total_complete);
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		dst->io_bytes[i]	= le64_to_cpu(src->io_bytes[i]);
 		dst->runtime[i]		= le64_to_cpu(src->runtime[i]);
 	}
@@ -656,7 +657,7 @@ static void convert_gs(struct group_run_stats *dst, struct group_run_stats *src)
 {
 	int i;
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		dst->max_run[i]		= le64_to_cpu(src->max_run[i]);
 		dst->min_run[i]		= le64_to_cpu(src->min_run[i]);
 		dst->max_bw[i]		= le64_to_cpu(src->max_bw[i]);
@@ -667,6 +668,7 @@ static void convert_gs(struct group_run_stats *dst, struct group_run_stats *src)
 
 	dst->kb_base	= le32_to_cpu(src->kb_base);
 	dst->groupid	= le32_to_cpu(src->groupid);
+	dst->unified_rw_rep	= le32_to_cpu(src->unified_rw_rep);
 }
 
 static void handle_ts(struct fio_client *client, struct fio_net_cmd *cmd)
@@ -687,6 +689,7 @@ static void handle_ts(struct fio_client *client, struct fio_net_cmd *cmd)
 
 	client_ts.members++;
 	client_ts.groupid = p->ts.groupid;
+	client_ts.unified_rw_rep = p->ts.unified_rw_rep;
 
 	if (++sum_stat_nr == sum_stat_clients) {
 		strcpy(client_ts.name, "All clients");
@@ -763,7 +766,7 @@ static void convert_jobs_eta(struct jobs_eta *je)
 	je->m_iops		= le32_to_cpu(je->m_iops);
 	je->t_iops		= le32_to_cpu(je->t_iops);
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		je->rate[i]	= le32_to_cpu(je->rate[i]);
 		je->iops[i]	= le32_to_cpu(je->iops[i]);
 	}
@@ -786,7 +789,7 @@ static void sum_jobs_eta(struct jobs_eta *dst, struct jobs_eta *je)
 	dst->m_iops		+= je->m_iops;
 	dst->t_iops		+= je->t_iops;
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < DDIR_RWDIR_CNT; i++) {
 		dst->rate[i]	+= je->rate[i];
 		dst->iops[i]	+= je->iops[i];
 	}
@@ -1037,7 +1040,7 @@ static int fio_client_timed_out(void)
 	struct timeval tv;
 	int ret = 0;
 
-	gettimeofday(&tv, NULL);
+	fio_gettime(&tv, NULL);
 
 	flist_for_each_safe(entry, tmp, &client_list) {
 		client = flist_entry(entry, struct fio_client, list);
@@ -1061,7 +1064,7 @@ int fio_handle_clients(void)
 	struct pollfd *pfds;
 	int i, ret = 0, retval = 0;
 
-	gettimeofday(&eta_tv, NULL);
+	fio_gettime(&eta_tv, NULL);
 
 	pfds = malloc(nr_clients * sizeof(struct pollfd));
 
@@ -1095,7 +1098,7 @@ int fio_handle_clients(void)
 		do {
 			struct timeval tv;
 
-			gettimeofday(&tv, NULL);
+			fio_gettime(&tv, NULL);
 			if (mtime_since(&eta_tv, &tv) >= 900) {
 				request_client_etas();
 				memcpy(&eta_tv, &tv, sizeof(tv));
